/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.services.rel

import com.persagy.server.services.base.RService
import com.persagy.server.datacenter.models.entities.rel.RShaftThroughShaft
import com.persagy.server.datacenter.models.entities.rel.RShaftThroughShaftOtherList
import com.persagy.server.services.objects.EquipmentService
import com.persagy.base.extensions.toJson
import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.service.SPageContext
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import org.json.JSONObject

/**
 * 竖井的贯通关系服务
 *
 * @author 付国宝
 */
object RShaftThroughShaftService : RService<RShaftThroughShaft>(SMybatisDao(RShaftThroughShaft::class.java)) {

    /**
     * 创建竖井的贯通关系
     *
     * @param   rFlThroughFl    竖井的贯通关系对象
     */
    fun create(rFlThroughFl: RShaftThroughShaft): SCreateResponse<RShaftThroughShaft> {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            rFlThroughFl.projectId = projectId
            val isInsert = insert(rFlThroughFl)
            if (isInsert){
                val jsonObject = JSONObject()
                jsonObject.put("type", "11")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rFlThroughFl.shaftId)
                jsonObject.put("obj_type", "Sh2Sh")
                jsonObject.put("linkId",rFlThroughFl.shaftOtherId)

                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "Sh2Sh", "11", IdUtils.uuidCreate(),  rFlThroughFl.shaftId!!,
//                        rFlThroughFl.shaftOtherId!!),false)
                SCreateResponse(SResponseType.success)
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Tenant create()

    /**
     * 根据对象删除
     *
     * @param   rFlThroughFl    关系对象实例
     */
    fun deletes(rFlThroughFl: RShaftThroughShaft): SBaseResponse {
        return try {
            val projectId = SPageContext.getHeader("projectId")
            val isDelete = delete(rFlThroughFl)

            val rShaftThroughShaft = RShaftThroughShaft()
            rShaftThroughShaft.shaftId = rFlThroughFl.shaftOtherId
            rShaftThroughShaft.shaftOtherId = rFlThroughFl.shaftId
            val isDeletes = delete(rShaftThroughShaft)
            /** 删除成功 */
            if (isDelete||isDeletes) {

                try {
                    /** 处理关系标记 */
                    val listType = ArrayList<String>()
                    listType.add("sh2sh")
                    for (type in listType){
                        val entity = EquipmentService.relationTypeProjectservice.select(SFilter.eq("projectId", projectId!!), SFilter.eq("relationType", type)).entity()
                        if (entity!=null){
                            entity.computationalState = 2
                            EquipmentService.relationTypeProjectservice.update(entity)
                        }
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                val jsonObject = JSONObject()
                jsonObject.put("type", "12")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rFlThroughFl.shaftId)
                jsonObject.put("obj_type", "Sh2Sh")
                jsonObject.put("linkId",rFlThroughFl.shaftOtherId)

                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "Sh2Sh", "12", IdUtils.uuidCreate(),  rFlThroughFl.shaftId!!,
//                        rFlThroughFl.shaftOtherId!!),false)
                val sBaseResponse = SBaseResponse(SResponseType.success)
                sBaseResponse.message = rFlThroughFl.toJson()
                sBaseResponse
            } else {
                SBaseResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Tenant deletes()

    /**
     * 创建竖井的贯通关系
     *
     * @param   rFloorThroughFloorOtherList    竖井和竖井列表关系对象
     */
    fun createShaft(rFloorThroughFloorOtherList: RShaftThroughShaftOtherList): SCreateResponse<RShaftThroughShaft> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()

            try {
                delete("floorId = '${rFloorThroughFloorOtherList.shaftId}'")
                delete("floorOtherId = '${rFloorThroughFloorOtherList.shaftId}'")
            } catch (e: Exception) {
                e.printStackTrace()
            }
            val projectId = SPageContext.getHeader("projectId")
            if (rFloorThroughFloorOtherList.shaftOtherIdList!=null&&rFloorThroughFloorOtherList.shaftOtherIdList!!.isNotEmpty()){
                for (floorOtherId in rFloorThroughFloorOtherList.shaftOtherIdList!!){
                    val rFlThroughFl = RShaftThroughShaft()
                    rFlThroughFl.shaftId = rFloorThroughFloorOtherList.shaftId
                    rFlThroughFl.shaftOtherId = floorOtherId
                    rFlThroughFl.projectId = projectId
                    val replace = replace(rFlThroughFl)
                    if (replace){
                        successList.add(rFlThroughFl)
                    }else{
                        failureList.add(rFlThroughFl)
                    }
                }
                if (successList.size>0){
                    val jsonObject = JSONObject()
                    jsonObject.put("type", "13")
                    jsonObject.put("projectId", projectId)
                    jsonObject.put("id", rFloorThroughFloorOtherList.shaftId)
                    jsonObject.put("obj_type", "Sh2Sh")
                    jsonObject.put("linkId",successList.toJson())
                    /** 发送消息 */
                    messageHandler.sendMessage(jsonObject.toString())
//                    kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "Sh2Sh", "13", IdUtils.uuidCreate(),  rFloorThroughFloorOtherList.shaftId!!, null),false)
                    val sCreateResponse = SCreateResponse<RShaftThroughShaft>(SResponseType.success)
                    if (failureList.size>0){
                        sCreateResponse.message ="失败对象："+failureList.toJson()
                    }
                    sCreateResponse
                }else{
                    SCreateResponse(SResponseType.failure)
                }
            }else{
                SCreateResponse(SResponseType.success)
            }

        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Tenant createShaft()


    /**
     * 创建竖井的贯通关系
     *
     * @param   rFloorThroughFloorOtherList    竖井和竖井列表关系对象
     */
    fun createShaftAll(rFloorThroughFloorOtherList: RShaftThroughShaftOtherList): SCreateResponse<RShaftThroughShaft> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()

            try {
//                delete("floorId = '${rFloorThroughFloorOtherList.shaftId}'")
//                delete("floorOtherId = '${rFloorThroughFloorOtherList.shaftId}'")
            } catch (e: Exception) {
                e.printStackTrace()
            }
            val projectId = SPageContext.getHeader("projectId")
            if (rFloorThroughFloorOtherList.shaftOtherIdList!=null&&rFloorThroughFloorOtherList.shaftOtherIdList!!.isNotEmpty()){
                for (floorOtherId in rFloorThroughFloorOtherList.shaftOtherIdList!!){
                    val rFlThroughFl = RShaftThroughShaft()
                    rFlThroughFl.shaftId = rFloorThroughFloorOtherList.shaftId
                    rFlThroughFl.shaftOtherId = floorOtherId
                    rFlThroughFl.projectId = projectId
                    val replace = replace(rFlThroughFl)
                    if (replace){
                        successList.add(rFlThroughFl)
                    }else{
                        failureList.add(rFlThroughFl)
                    }
                }
                if (successList.size>0){

                    try {
                        /** 处理关系标记 */
                        val listType = ArrayList<String>()
                        listType.add("sh2sh")
                        for (type in listType){
                            val entity = EquipmentService.relationTypeProjectservice.select(SFilter.eq("projectId", projectId!!), SFilter.eq("relationType", type)).entity()
                            if (entity!=null){
                                entity.computationalState = 2
                                EquipmentService.relationTypeProjectservice.update(entity)
                            }
                        }
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }

                    val jsonObject = JSONObject()
                    jsonObject.put("type", "14")
                    jsonObject.put("projectId", projectId)
                    jsonObject.put("id", rFloorThroughFloorOtherList.shaftId)
                    jsonObject.put("obj_type", "Sh2Sh")
                    jsonObject.put("linkId",successList.toJson())
                    /** 发送消息 */
                    messageHandler.sendMessage(jsonObject.toString())
//                    kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "Sh2Sh", "14", IdUtils.uuidCreate(),  rFloorThroughFloorOtherList.shaftId!!, null),false)
                    val sCreateResponse = SCreateResponse<RShaftThroughShaft>(SResponseType.success)
                    if (failureList.size>0){
                        sCreateResponse.message ="失败对象："+failureList.toJson()
                    }
                    sCreateResponse
                }else{
                    SCreateResponse(SResponseType.failure)
                }
            }else{
                SCreateResponse(SResponseType.success)
            }

        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Tenant createShaftAll()


} // Object RShaftThroughShaftService()