/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.controllers

import com.persagy.server.datacenter.models.entities.objects.Equipment
import com.persagy.server.datacenter.models.entities.objects.GeneralSystem
import com.persagy.server.datacenter.models.entities.objects.Shaft
import com.persagy.server.models.objects.FloorCompon
import com.persagy.server.models.requests.ShaftSpaceVertical
import com.persagy.server.service.EquipmentZoneService
import com.persagy.server.service.objects.EquipmentService
import com.persagy.server.service.objects.GeneralSystemService
import com.persagy.server.service.objects.ShaftService
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 竖井接口
 *
 * @author 张维新
 */
@Tag(name = "竖井相关接口")
@RestController
@RequestMapping("/shaft")
class ShaftController {
    @Autowired
    lateinit var equipmentZoneService: EquipmentZoneService
    /**
     * 查询没有和当前竖井绑定的系统
     *
     * @param   shaftId     竖井id
     */
   @Operation(summary="查询没有和当前竖井绑定的系统", description= "")
    @PostMapping(value = ["/unshaft-in-sys"])
    fun unShaftSys(shaftId: String,@RequestBody request: SQueryRequest): SQueryResponse<GeneralSystem> {
        return GeneralSystemService.unShaftSys(shaftId, request)
    } // Function unSpaceSys()

    /**
     * 查询没有和当前竖井绑定的设备
     */
   @Operation(summary="查询没有和当前竖井绑定的设备", description ="")
    @PostMapping(value = ["/unshaft-in-eq"])
    fun  unShaftEq( shaftId: String,@RequestBody request: SQueryRequest): SQueryResponse<Equipment> {
        return EquipmentService.pageQueryShaftEquip(shaftId,request)
    } // Function unSpaceEq()

   @Operation(summary="查询没有和当前竖井贯通的其它竖井", description ="")
    @PostMapping(value = ["/unshaft-through-shaft"])
    fun  unShaftThroughShaft(shaftId: String,@RequestBody request: SQueryRequest): SQueryResponse<Shaft> {
        return ShaftService.unShaftThroughShaft(shaftId,request)
    } // Function unSpaceEq()

   @Operation(summary="查询竖井关联的空间垂直交通关系", description ="")
    @PostMapping(value = ["/shaft-vertical-space"])
    fun shaftSpaceVertical(@RequestBody request: ShaftSpaceVertical): SQueryResponse<FloorCompon>{
//        equipmentZoneService.similars()
        return ShaftService.shaftSpace(request)
    } // Function shaftSpaceVertical()

    /**
     * 计算 和当前竖井有关联的空间的垂直交通关系
     */
   @Operation(summary="计算 和当前竖井有关联的空间的垂直交通关系", description ="")
    @PostMapping(value = ["/vertical-space"])
    fun shaftVerticalSp(@RequestBody request: ShaftSpaceVertical): SQueryResponse<FloorCompon>{
        return ShaftService.spVerticalSp(request)
    } // Fun


}  // Class SpaceController