/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.scheduler.controllers

import com.persagy.database.SFilter
import com.persagy.server.scheduler.models.CommitTaskRequest
import com.persagy.server.scheduler.models.DispatchTaskResponse
import com.persagy.server.scheduler.models.TaskEntity
import com.persagy.server.scheduler.pojo.Command
import com.persagy.server.scheduler.server.DispatchTask
import com.persagy.server.scheduler.service.BaseDataService
import com.persagy.server.scheduler.service.TasksService
import com.persagy.server.scheduler.utils.CommonUtil
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController


@Api(tags = ["Task接口"])
@RestController
@RequestMapping("/task")
open class TaskController{
    companion object {
        // 日志
        private val logger = LoggerFactory.getLogger(TaskController::class.java)
    } // Companion object

    @Autowired
    lateinit var tasksService : TasksService

    /**
     * 被动派发Task任务
     *
     * @param   request     请求
     * @return  派发结果应答信息
     */
    @ApiOperation(value="派发Task任务", notes="输入为下载模型文件的md5集合")
    @PostMapping(value = ["/dispatch"])
    fun dispatch(@RequestBody request: ArrayList<String>): SBaseResponse {
        val allTaskMap = tasksService.getAllTaskMapByStatus(arrayListOf(0, 1))
        if(allTaskMap.isEmpty()){
            return SBaseResponse()
        }
        // 匹配到MD5的任务
        val taskEntity = tasksService.getTaskByMD5(request, allTaskMap)
        val dispatchTaskResponse = DispatchTaskResponse()
        dispatchTaskResponse.result = SResponseType.success
        if(taskEntity == null){
//            dispatchTaskResponse.tasks.add(allTaskMap[0]!!)
            dispatchTaskResponse.tasks.add(allTaskMap.values.iterator().next())
        }else
            dispatchTaskResponse.tasks.add(taskEntity)
        return dispatchTaskResponse
    } // Function update()

    /**
     * 客户端确认接受Task任务
     *
     * @param   request     请求
     * @return  结果应答信息
     */
    @ApiOperation(value="确认Task任务", notes="输入为任务Id, LastClient, 暂时只支持单个任务的确认")
    @PostMapping(value = ["/confirm"])
    fun confirmDispatch(@RequestBody request: TaskEntity): SBaseResponse {
        val response = SBaseResponse()
        request.status = 2
        val updateResult = BaseDataService.taskService.update(request, arrayListOf("status", "lastClient"), SFilter.inList("status", (arrayListOf(0, 1))))
        if(updateResult){
            response.result = SResponseType.success
        }
        return response
    } // Function update()

    /**
     * 客户端提交Task任务结果
     *
     * @param   request     请求
     * @return  结果应答信息
     */
    @ApiOperation(value="提交Task任务结果", notes="输入为任务Id, TaskResultJson, ClientInfo, 暂时只支持单个任务的提交")
    @PostMapping(value = ["/commit"])
    fun commit(@RequestBody request: CommitTaskRequest): SBaseResponse {
        val response = SBaseResponse()
        if(request.command == null)
            return response
        when(request.command){
            Command.CommandError -> {
                request.status = 4
            }
            Command.DownloadError -> {
                request.status = 3
            }
            Command.TaskSuccess -> {
                request.status = 5
            }
        }
        DispatchTask.processResult(request, request.resultJson?:"")
        val updateResult = BaseDataService.taskService.update(request, arrayListOf("status", "resultJson", "clientInfo"), SFilter.Companion.inList("status", arrayListOf(0, 1, 2, 3, 4)))
        if(updateResult)
            response.result = SResponseType.success
        return response
    } // Function update()


    /**
     * 创建Task
     */
    @ApiOperation(value = "创建Task", notes = "")
    @PostMapping(value = ["/create"])
    fun create(@RequestBody taskEntity: SCreateRequest<TaskEntity>): SCreateResponse<TaskEntity> {
        //logger.debug("download =${taskEntity.toJson()}")
//        val response = taskService.createList(taskEntity)
        taskEntity.content.forEach {
            if(it.id == null){
                it.id = CommonUtil.generateUUID("", true)
            }
        }
        return BaseDataService.taskService.createList(taskEntity)
    } // Function create()

    /**
     * 更新Task
     *
     * @param   request     更新请求
     * @return  更新结果应答信息
     */
    @ApiOperation(value="更新Task", notes="")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<TaskEntity>): SBaseResponse {
        return BaseDataService.taskService.updateList(request)
    } // Function update()

    /**
     * 查询Task
     *
     * @param   request     查询请求
     * @return  查询结果应答信息
     */
    @ApiOperation(value="查询Task", notes="")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<TaskEntity> {
        return BaseDataService.taskService.pageQuery(request)
    } // Function query()


    /**
     * 删除Task
     *
     * @param   idList      TaskEntity ID列表
     * @return  删除结果应答信息
     */
    @ApiOperation(value = "删除Task", notes = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<TaskEntity>): SBaseResponse {
        return BaseDataService.taskService.deleteByKeysList(idList)
    } // Function delete()
}